
; Eines der vielen Skripte die den Umgang mit ASM vereinfachen wollen.
; Dieses mal allerdings ohne Debugmglichkeiten wie bei AssembleIt.
; Es ist auf einfache und schnelle Bedienung ausgelegt.

#include-once
#include "FASM.au3"

#region - ASM - Kern
; Globale Variablen
Global $Fasm

; Struct Array
Global $___avOP[1]

; DLLs
Global $___ASM_DLL_USER32

; #FUNCTION# ====================================================================================================================
; Name...........: _ASM_Startup
; Description ...: Initialisiert FASM, Ld Dlls
; Syntax.........: _ASM_Startup()
; Parameters ....:
; Return values .:
; Author ........: Mars
; Modified.......:
; Remarks .......:
; Related .......: _ASM_Shutdown
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _ASM_Startup()

	$Fasm = FasmInit()

	; Erstmal ein Reset
	FasmReset($Fasm)

	; DLLs starten
	$___ASM_DLL_USER32 = DllOpen('USER32.DLL')

EndFunc

; #FUNCTION# ====================================================================================================================
; Name...........: _ASM_Shutdown
; Description ...: Gibt die Ressourcen wieder frei, beendet FASM
; Syntax.........: _ASM_Shutdown()
; Parameters ....:
; Return values .:
; Author ........: Mars
; Modified.......:
; Remarks .......:
; Related .......: _ASM_Startup
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _ASM_Shutdown()

	; Structs leeren
	For $i = 1 To $___avOP[0] Step 1
		$___avOP[$i] = 0
	Next

	; DLLs schlieen
	DllClose($___ASM_DLL_USER32)

	FasmExit($Fasm)
	FasmShutdown()

EndFunc

; #FUNCTION# ====================================================================================================================
; Name...........: _ASM_Add_OP
; Description ...: Fgt OP-Codes zum Struct Array hinzu
; Syntax.........: _ASM_Add_OP($a)
; Parameters ....: $a - Dieser Parameter ermglicht das kompakte Aufrufen der ASM Funktion die hinzugefgt werden soll.
; Return values .: Aktueller OP-Index. (Zufriff auf ASM Ops ber diesen Index mglich)
; Author ........: Mars
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _ASM_Add_OP($a)
	Local $vOP = __ASM_GetStruct()
	Local $u = UBound($___avOP)
	ReDim $___avOP[$u+1]
	$___avOP[0] += 1
	$___avOP[$u] = $vOP
	Return $___avOP[0]
EndFunc

; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name...........: __ASM_GetStruct
; Description ...: Verpackt den in Fasm befindlichen Code in eine DllStruct die per CallWindowProc aufgerufen werden kann.
; Syntax.........: __ASM_GetStruct()
; Parameters ....:
; Return values .: $vStruct - Die DllStruct beinhaltet den ASM-Code
; Author ........: Mars
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func __ASM_GetStruct()
	Local $vStruct = __ASM_ToStruct(FasmGetBinary($Fasm))
	FasmReset($Fasm)
	Return $vStruct
EndFunc   ;==>_Get_ASM_Struct

; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name...........: __ASM_ToStruct
; Description ...: Verpackt den im Parameter enthaltenen Code in eine DLLStruct
; Syntax.........: __ASM_ToStruct($ASM)
; Parameters ....: $ASM  - ASM Code
; Return values .: $vStr - Die DllStruct beinhaltet den Code aus $ASM
; Author ........: Mars
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func __ASM_ToStruct($ASM)
	Local $vStr = DllStructCreate('byte[' & StringLen($ASM) / 2 - 1 & ']')
	DllStructSetData($vStr, 1, $ASM)
	Return $vStr
EndFunc   ;==>_AsmToStruct

; #FUNCTION# ====================================================================================================================
; Name...........: _ASM_Call
; Description ...: Ruft eine ASM Func mit 4 integer Parametern auf
; Syntax.........: _ASM_Call($iOP, $1, $2 = 0, $3 = 0, $4 = 0)
; Parameters ....: $iOP = Nummer der zu verwendenden OP-Codes
; Return values .:
; Author ........: Mars
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _ASM_Call($iOP, $1, $2 = 0, $3 = 0, $4 = 0)
	Return DllCall($___ASM_DLL_USER32, 'ptr', 'CallWindowProcW', 'ptr', DllStructGetPtr($___avOP[$iOP]), 'int', $1, 'int', $2, 'int', $3, 'int', $4)
EndFunc

; #FUNCTION# ====================================================================================================================
; Name...........: _
; Description ...: Vereinfachung von FasmAdd (damit man nicht immer so viel schreiben muss)
; Syntax.........: _($a)
; Parameters ....: $a - ASM Code
; Return values .:
; Author ........: Mars
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _($a)
	FasmAdd($Fasm, $a)
EndFunc
#endregion - ASM Kern
;endregion - ASM Kern